; Emulation of 4MHz SCMP / Mk14 on 20MHz PIC16F876 / PIC16F877
;
; (c) 2007 K Orton
;
; This emulation is cycle perfect for programs running in RAM.
; SA interrupts are not implemented.
; Illegal SCMP insructions cause an SCMP reset.
;
; PORTA is miscellaneous and keyboard row sense inputs:
;
;   RA5     RA4     RA3     RA2     RA1     RA0
;  /SCMP   Serial   KBD7*   KBD6*   KBD5*   KBD4*
;  reset   input
;
; *As per Mk14 data bus assignment
;  Note: SCMP reset input is active low.
;
; PORTB is active high display segment drives:
;
;  RB7  RB6  RB5  RB4  RB3  RB2  RB1  RB0
;  dp   g    f    e    d    c    b    a
;
; PORTC is active high display/keyboard column drives:
;
;  RC7  RC6  RC5  RC4  RC3  RC2  RC1  RC0
;  left ...  ...  ...  ...  ...  ...  right
;  
; PORTD and PORTE bring out SCMP flags on 16F877:
;
;  RD7  RD6  RD5  RD4  RD3  RD2  RD1  RD0
;  SIN  -    SB   SA   F3*  F2   F1   F0
;
;  RE2  RE1  RE0
;  -    -    SOUT
;
; *Actually SCMP IE status bit but available for use as a flag.
;
; Ports RD6, RE1 and RE2 are inputs - recommend they are tied
;  high with 10k resistors.
;
; Compile time switch - serial download supported
;
SERIAL	EQU	0

; PIC constants
W	EQU	0
F	EQU	1
INDF	EQU	H'0000'
PCL	EQU	H'0002'
STATUS	EQU	H'0003'
FSR	EQU	H'0004'
PORTA	EQU	H'0005'
PORTB	EQU	H'0006'
PORTC	EQU	H'0007'
PORTD	EQU	H'0008'
PORTE	EQU	H'0009'
PCLATH	EQU	H'000A'
INTCON	EQU	H'000B'
OPTREG	EQU	H'0081'
TRISA	EQU	H'0085'
TRISB	EQU	H'0086'
TRISC	EQU	H'0087'
TRISD	EQU	H'0088'
TRISE	EQU	H'0089'
ADCON1	EQU	H'009F'
EEDATA	EQU	H'010C'
EEADR	EQU	H'010D'
EECON1	EQU	H'018C'
EECON2	EQU	H'018D'
RD	EQU	0
WR	EQU	1
WREN	EQU	2
PGD	EQU	7
C	EQU	0
DC	EQU	1
Z	EQU	2
RP0	EQU	5
RP1	EQU	6
IRP	EQU	7
CFG	EQU	H'3D3A'

	__CONFIG	CFG

; Program constants
; Registers 32..95 in every bank are used
;  to hold RAM image (total 256 bytes)
ACC	EQU	H'70'
EXT	EQU	H'71'
STAT	EQU	H'72'
P0HI	EQU	H'73'
P0LO	EQU	H'74'
P1HI	EQU	H'75'
P1LO	EQU	H'76'
P2HI	EQU	H'77'
P2LO	EQU	H'78'
P3HI	EQU	H'79'
P3LO	EQU	H'7A'
OPRND	EQU	H'7B'
EAHI	EQU	H'7C'
EALO	EQU	H'7D'
DATUM	EQU	H'7E'
TMP	EQU	H'7F'
IE	EQU	3
OV	EQU	6
CYL	EQU	7
SER	EQU	4
RST	EQU	5

; Read ROM macro part 1 (9)
RDROM1	MACRO	PHI,PLO
	MOVF	PHI,W
	ANDLW	B'00000001'
	ADDLW	HIGH SCIOS
	MOVWF	PCLATH
	MOVF	PLO,W
	MOVWF	PCL
	ENDM

; Read ROM macro part 2 (15)
; Read data returned in DST and W
RDROM2	MACRO	ROMSUB,DST,LATH
	CALL	ROMSUB
	MOVWF	DST
	MOVLW	LATH
	MOVWF	PCLATH
	MOVF	DST,W
	ENDM

; Read RAM macro (10)
; Read data returned in W
RDRAM	MACRO	PLO
	BCF	STATUS,IRP
	BTFSC	PLO,7
	BSF	STATUS,IRP
	MOVF	PLO,W
	ANDLW	B'00111111'
	ADDLW	H'20'
	BTFSC	PLO,6
	IORLW	B'10000000'
	MOVWF	FSR
	MOVF	INDF,W
	ENDM

; Write RAM macro (11)
WRRAM	MACRO	PLO,SCE
	BCF	STATUS,IRP
	BTFSC	PLO,7
	BSF	STATUS,IRP
	MOVF	PLO,W
	ANDLW	B'00111111'
	ADDLW	H'20'
	BTFSC	PLO,6
	IORLW	B'10000000'
	MOVWF	FSR
	MOVF	SCE,W
	MOVWF	INDF
	ENDM

; Long jump to home page macro (4)
HOME	MACRO	DST
	MOVLW	HIGH OPVEC
	MOVWF	PCLATH
	GOTO	DST
	ENDM

; Long jump to page 0 macro (3)
LJMP0	MACRO	DST
	CLRF	PCLATH
	GOTO	DST
	ENDM

; Exchange byte macro (6)
XCHNG	MACRO	A,B
	MOVF	A,W
	MOVWF	TMP
	MOVF	B,W
	MOVWF	A
	MOVF	TMP,W
	MOVWF	B
	ENDM

; Binary add macro (16)
ADD	MACRO	SCE
	MOVF	SCE,W
	XORWF	ACC,W
	XORLW	H'80'
	MOVWF	TMP
	BTFSC	STAT,CYL
	INCFSZ	ACC,F
	BCF	STAT,CYL
	MOVF	SCE,W
	ADDWF	ACC,F
	BTFSC	STATUS,C
	BSF	STAT,CYL
	XORWF	ACC,W
	ANDWF	TMP,F
	BCF	STAT,OV
	BTFSC	TMP,7
	BSF	STAT,OV
	ENDM

; Decimal add macro (14)
DAD	MACRO	SCE
	MOVLW	H'66'
	ADDWF	SCE,F
	BTFSC	STAT,CYL
	INCF	ACC,F
	RLF	STAT,F
	MOVF	SCE,W
	ADDWF	ACC,F
	RRF	STAT,F
	MOVLW	H'06'
	BTFSS	STATUS,DC
	SUBWF	ACC,F
	MOVLW	H'60'
	BTFSS	STAT,CYL
	SUBWF	ACC,F
	ENDM

; Increment PC macro (3)
INCPC	MACRO
	INCF	P0HI,F
	INCFSZ	P0LO,F
	DECF	P0HI,F
	ENDM

; Get operand macro (17)
; Operand returned in OPRND and W
GOP	MACRO	P0SUB,LATH
	LOCAL	L1
	LOCAL	L2
	INCPC
	BTFSC	P0HI,3
	GOTO	L1
	RDROM2	P0SUB,OPRND,LATH
	GOTO	L2
L1	RDRAM	P0LO
	MOVWF	OPRND
L2
	ENDM

; Get effective address macro (15)
; Operand expected in OPRND and W
; Effective address returned in EAHI, EALO
GEA	MACRO	PHI,PLO
	LOCAL	L1
	LOCAL	L2
	XORLW	H'80'
	BTFSC	STATUS,Z
	GOTO	L1
	NOP
	GOTO	L2
L1	MOVF	EXT,W
	MOVWF	OPRND
L2	MOVF	PHI,W
	MOVWF	EAHI
	BTFSC	OPRND,7
	DECF	EAHI,F
	MOVF	OPRND,W
	ADDWF	PLO,W
	MOVWF	EALO
	BTFSC	STATUS,C
	INCF	EAHI,F
	ENDM

; Effective address to PC macro (32)
; For home page use only
; Exits directly to EXEC
GEAPC	MACRO	PHI,PLO
	LOCAL	L1
	GOP	P0PT1,HIGH OPVEC
	XORLW	H'80'
;	BTFSS	STATUS,Z			; Bug fix: JMP H80 does not use EXT
	NOP
	GOTO	L1
	NOP
	MOVF	PHI,W
	MOVWF	P0HI
	BTFSC	EXT,7
	DECF	P0HI,F
	MOVF	EXT,W
	ADDWF	PLO,W
	MOVWF	P0LO
	BTFSC	STATUS,C
	INCF	P0HI,F
	GOTO	EXEC
L1	MOVF	PHI,W
	MOVWF	P0HI
	BTFSC	OPRND,7
	DECF	P0HI,F
	MOVF	OPRND,W
	ADDWF	PLO,W
	MOVWF	P0LO
	BTFSC	STATUS,C
	INCF	P0HI,F
	GOTO	EXEC
	ENDM

; Read memory macro (18)
; Read data returned in DATUM and W
RDMEM	MACRO	PHI,PLO,DSUB,PSUB,LATH
	LOCAL	L1
	LOCAL	L2
	LOCAL	L3
	BTFSS	PHI,3
	GOTO	L2
	BTFSS	PHI,1
	GOTO	L1
	RDRAM	PLO
	MOVWF	DATUM
	NOP
	GOTO	L3
L1	SWAPF	PORTA,W
	IORLW	B'00001111'
	MOVWF	DATUM
	CALL	DSUB
	CALL	DSUB
	GOTO	L3
L2	RDROM2	PSUB,DATUM,LATH
L3
	ENDM

; Auto indexed read memory macro (59)
; For home page use only
; Read data returned in DATUM and W
RDMEMA	MACRO	PHI,PLO,PSUB
	LOCAL	L1
	LOCAL	L2
	GOP	P0PT1,HIGH OPVEC
	GEA	PHI,PLO
	BTFSC	OPRND,7
	GOTO	L1
	RDMEM	PHI,PLO,DEL4,PSUB,HIGH OPVEC
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	MOVF	DATUM,W
	GOTO	L2
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	RDMEM	PHI,PLO,DEL4,PSUB,HIGH OPVEC
	NOP
	NOP
L2
	ENDM

; Auto indexed add memory macro (76)
; For home page use only
; Exits directly to EXEC
ADMEMA	MACRO	PHI,PLO,PSUB
	LOCAL	L1
	GOP	P0PT1,HIGH OPVEC
	GEA	PHI,PLO
	NOP
	BTFSC	OPRND,7
	GOTO	L1
	NOP
	RDMEM	PHI,PLO,DEL4,PSUB,HIGH OPVEC
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	ADD	DATUM
	GOTO	EXEC
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	RDMEM	PHI,PLO,DEL4,PSUB,HIGH OPVEC
	ADD	DATUM
	GOTO	EXEC
	ENDM

; Write memory macro (22)
; For page 0 use only
WRMEM	MACRO	PHI,PLO
	LOCAL	L1
	LOCAL	L2
	LOCAL	L3
	BTFSS	PHI,3
	GOTO	L1
	BTFSS	PHI,1
	GOTO	L2
	WRRAM	PLO,ACC
	MOVF	PLO,W
	MOVWF	TMP
	BTFSS	P0HI,3
	GOTO	CHKEE
	NOP
	RETURN
L1	CALL	DEL4X
	CALL	DEL4X
	CALL	DEL4X
	CALL	DEL4X
	NOP
	RETURN
L2	CALL	L3
	CLRF	PORTB
	NOP
	MOVWF	PORTC
	MOVF	ACC,W
	MOVWF	PORTB
	RETURN
L3	MOVLW	HIGH ($+4)
	MOVWF	PCLATH
	MOVF	PLO,W
	ANDLW	B'00001111'
	IF	HIGH($) != HIGH($+D'16')
	ERROR	"PAGING ERROR"
	ENDIF
	ADDWF	PCL,F
	RETLW	B'00000001'
	RETLW	B'00000010'
	RETLW	B'00000100'
	RETLW	B'00001000'
	RETLW	B'00010000'
	RETLW	B'00100000'
	RETLW	B'01000000'
	RETLW	B'10000000'
	RETLW	0
	RETLW	0
	RETLW	0
	RETLW	0
	RETLW	0
	RETLW	0
	RETLW	0
	RETLW	0
	ENDM

; Auto indexed write memory macro (68)
; For page 0 use only
; Exits directly to EXEC
WRMEMA	MACRO	PHI,PLO,WRSUB
	LOCAL	L1
	GOP	P0PT1X,0
	GEA	PHI,PLO
	BTFSC	OPRND,7
	GOTO	L1
	CALL	WRSUB
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	HOME	PAD2
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	CALL	WRSUB
	NOP
	HOME	EXEC
	ENDM

; PIC reset
	ORG	0
	BSF	STATUS,RP0
	MOVLW	B'00000110'
	MOVWF	ADCON1
	MOVLW	B'11010111'
	MOVWF	OPTREG
	CLRF	INTCON
	MOVLW	B'11111111'
	MOVWF	TRISA
	MOVLW	B'00000000'
	MOVWF	TRISB
	MOVWF	TRISC
	MOVLW	B'11110000'
	MOVWF	TRISD
	MOVLW	B'00000110'	; Bug fix: TRISE initialisation
	MOVWF	TRISE
	BCF	STATUS,RP0
	CLRF	PORTB
	CLRF	PORTC
	CLRF	PORTD
	MOVLW	B'00000001'
	MOVWF	PORTE
	CALL	LDEE
	IF	SERIAL
	CALL	RXFILE
	ENDIF		

; SCMP reset
RESET	CLRF	ACC
	CLRF	EXT
	CLRF	STAT
	CLRF	P0HI
	CLRF	P0LO
	CLRF	P1HI
	CLRF	P1LO
	CLRF	P2HI
	CLRF	P2LO
	CLRF	P3HI
	CLRF	P3LO
	CLRF	PORTD		; Bug fix: SC/MP reset clears flags
	CLRF	PORTE
L0	BTFSS	PORTA,RST
	GOTO	L0
	HOME	EXEC

; Load EEPROM data
; Uses TMP
LDEE	BSF	STATUS,RP1
	CLRF	TMP
L1	MOVF	TMP,W
	MOVWF	EEADR
	BSF	STATUS,RP0
	BCF	EECON1,PGD
	BSF	EECON1,RD
	BCF	STATUS,RP0
	WRRAM	TMP,EEDATA
	INCFSZ	TMP,F
	GOTO	L1
	BCF	STATUS,RP1
	RETURN

; Write EEPROM
; Address expected in TMP
; Data expected in ACC
WREE	BSF	STATUS,RP0
	BSF	STATUS,RP1
L2	BTFSC	EECON1,WR
	GOTO	L2
	BCF	STATUS,RP0
	MOVF	TMP,W
	MOVWF	EEADR
	MOVF	ACC,W
	MOVWF	EEDATA
	BSF	STATUS,RP0
	BCF	EECON1,PGD
	BSF	EECON1,WREN
	MOVLW	H'55'
	MOVWF	EECON2
	MOVLW	H'AA'
	MOVWF	EECON2
	BSF	EECON1,WR
	BCF	EECON1,WREN
	BCF	STATUS,RP0
	BCF	STATUS,RP1
	RETURN

; Receive serial byte at 9600 baud,
; 8 data bits, no parity, 1 stop bit.
; Byte returned in EXT and W
; Uses ACC and TMP
RXSER	BTFSS	PORTA,RST
	GOTO	RESET
	BTFSC	PORTA,SER
	GOTO	RXSER
	MOVLW	D'86'
	MOVWF	TMP
L3	DECFSZ	TMP,F
	GOTO	L3
	BTFSC	PORTA,SER
	GOTO	RXSER
	MOVLW	8
	MOVWF	ACC
	MOVLW	H'AB'
	MOVWF	TMP
L4	DECFSZ	TMP,F
	GOTO	L4
L5	RRF	EXT,F
	BCF	EXT,7
	BTFSC	PORTA,SER
	BSF	EXT,7
	MOVLW	H'AB'
	MOVWF	TMP
L6	DECFSZ	TMP,F
	GOTO	L6
	DECFSZ	ACC,F
	GOTO	L5
	MOVF	EXT,W
	RETURN

; Receive hex pair
; Byte returned in STAT and W
RXHEXP	CALL	RXSER
	MOVLW	H'3A'
	SUBWF	EXT,F
	MOVLW	7
	BTFSC	EXT,7
	ADDWF	EXT,F
	MOVLW	3
	ADDWF	EXT,F
	SWAPF	EXT,W
	MOVWF	STAT
	CALL	RXSER
	MOVLW	H'3A'
	SUBWF	EXT,F
	MOVLW	7
	BTFSC	EXT,7
	ADDWF	EXT,F
	MOVF	EXT,W
	ADDLW	3
	ADDWF	STAT,W
	MOVWF	STAT
	RETURN

; Receive hex file
; Uses P0LO, EAHI, EALO
RXFILE	CALL	RXSER
	XORLW	H'3A'
	BTFSS	STATUS,Z
	GOTO	RXFILE
	CALL	RXHEXP
	MOVWF	P0LO
	MOVF	P0LO,F
	BTFSC	STATUS,Z
	GOTO	RESET
	CALL	RXHEXP
	MOVWF	EAHI
	CALL	RXHEXP
	MOVWF	EALO
	CALL	RXHEXP
L7	CALL	RXHEXP
	WRRAM	EALO,STAT
	INCF	EALO,F
	DECFSZ	P0LO,F
	GOTO	L7
	GOTO	RXFILE

; Transmit serial byte at 9600 baud,
; 8 data bits, no parity, 1 stop bit.
; Byte expected in EXT (destroyed)
; Uses ACC and TMP
TXSER	BCF	PORTE,0
	MOVLW	H'AB'
	MOVWF	TMP
L8	DECFSZ	TMP,F
	GOTO	L8
	NOP
	MOVLW	8
	MOVWF	ACC
L9	BTFSC	EXT,0
	GOTO	L10
	NOP
	BCF	PORTE,0
	GOTO	L11
L10	BSF	PORTE,0
	GOTO	L11
L11	MOVLW	H'AA'
	MOVWF	TMP
L12	DECFSZ	TMP,F
	GOTO	L12
	RRF	EXT,F
	DECFSZ	ACC,F
	GOTO	L9
	CALL	DEL4X
	BSF	PORTE,0
	RETURN

; Opcode functions
XAEX	XCHNG	ACC,EXT
	HOME	PAD3
XPAL0X	XCHNG	ACC,P0LO
	HOME	PAD8
XPAL1X	XCHNG	ACC,P1LO
	HOME	PAD8
XPAL2X	XCHNG	ACC,P2LO
	HOME	PAD8
XPAL3X	XCHNG	ACC,P3LO
	HOME	PAD8
XPAH0X	XCHNG	ACC,P0HI
	HOME	PAD8
XPAH1X	XCHNG	ACC,P1HI
	HOME	PAD8
XPAH2X	XCHNG	ACC,P2HI
	HOME	PAD8
XPAH3X	XCHNG	ACC,P3HI
	HOME	PAD8
LD0X	CALL	RD0X
	MOVWF	ACC
	HOME	PAD9
LD1X	CALL	RD1X
	MOVWF	ACC
	HOME	PAD9
LD2X	CALL	RD2X
	MOVWF	ACC
	HOME	PAD9
LD3X	CALL	RD3X
	MOVWF	ACC
	HOME	PAD9
ADD0X	GOP	P0PT1X,0
	GEA	P0HI,P0LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	ADD	DATUM
	HOME	PAD3
ADD1X	GOP	P0PT1X,0
	GEA	P1HI,P1LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	ADD	DATUM
	HOME	PAD3
ADD2X	GOP	P0PT1X,0
	GEA	P2HI,P2LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	ADD	DATUM
	HOME	PAD3
ADD3X	GOP	P0PT1X,0
	GEA	P3HI,P3LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	ADD	DATUM
	HOME	PAD3
CAD0X	CALL	RD0X
	COMF	DATUM,F
	ADD	DATUM
	HOME	PAD3
CAD1X	CALL	RD1X
	COMF	DATUM,F
	ADD	DATUM
	HOME	PAD3
CAD2X	CALL	RD2X
	COMF	DATUM,F
	ADD	DATUM
	HOME	PAD3
CAD3X	CALL	RD3X
	COMF	DATUM,F
	ADD	DATUM
	HOME	PAD3
AND0X	CALL	RD0X
	ANDWF	ACC,F
	HOME	PAD9
AND1X	CALL	RD1X
	ANDWF	ACC,F
	HOME	PAD9
AND2X	CALL	RD2X
	ANDWF	ACC,F
	HOME	PAD9
AND3X	CALL	RD3X
	ANDWF	ACC,F
	HOME	PAD9
OR0X	CALL	RD0X
	IORWF	ACC,F
	HOME	PAD9
OR1X	CALL	RD1X
	IORWF	ACC,F
	HOME	PAD9
OR2X	CALL	RD2X
	IORWF	ACC,F
	HOME	PAD9
OR3X	CALL	RD3X
	IORWF	ACC,F
	HOME	PAD9
XOR0X	CALL	RD0X
	XORWF	ACC,F
	HOME	PAD9
XOR1X	CALL	RD1X
	XORWF	ACC,F
	HOME	PAD9
XOR2X	CALL	RD2X
	XORWF	ACC,F
	HOME	PAD9
XOR3X	CALL	RD3X
	XORWF	ACC,F
	HOME	PAD9
DAD0X	CALL	RD0X
	CALL	DADDX
	HOME	PAD17
DAD1X	CALL	RD1X
	CALL	DADDX
	HOME	PAD17
DAD2X	CALL	RD2X
	CALL	DADDX
	HOME	PAD17
DAD3X	CALL	RD3X
	CALL	DADDX
	HOME	PAD17
LDIX	CALL	GOPX
	MOVWF	ACC
	HOME	PAD2
ANIX	GOP	P0PT1X,0
	ANDWF	ACC,F
	HOME	PAD6
ORIX	CALL	GOPX
	IORWF	ACC,F
	HOME	PAD2
XRIX	CALL	GOPX
	XORWF	ACC,F
	HOME	PAD2
DAEX	DAD	EXT
	HOME	PAD15
DAIX	CALL	GOPX
	DAD	OPRND
	HOME	PAD14
CAIX	GOP	P0PT1X,0
	COMF	OPRND,F
	ADD	OPRND
	HOME	EXEC
ILD0X	CALL	RD0X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
ILD1X	CALL	RD1X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
ILD2X	CALL	RD2X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
ILD3X	CALL	RD3X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
DLD0X	CALL	RD0X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
DLD1X	CALL	RD1X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
DLD2X	CALL	RD2X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
DLD3X	CALL	RD3X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	PAD4
ST0X	CALL	EA0X
	CALL	WREAX
	HOME	EXEC
ST1X	CALL	EA1X
	CALL	WREAX
	HOME	EXEC
ST2X	CALL	EA2X
	CALL	WREAX
	HOME	EXEC
ST3X	CALL	EA3X
	CALL	WREAX
	HOME	EXEC
ST1AX	WRMEMA	P1HI,P1LO,WR1X
ST2AX	WRMEMA	P2HI,P2LO,WR2X
ST3AX	WRMEMA	P3HI,P3LO,WR3X
DLYX	CALL	GOPX
	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	L16
	NOP
	NOP
L15	NOP
	NOP
	NOP
	CALL	DEL4X
	DECFSZ	ACC,F
	GOTO	L15
L16	MOVF	OPRND,F
	BTFSC	STATUS,Z
	GOTO	L19
	NOP
	NOP
L17	NOP
	NOP
	NOP
	MOVLW	H'E9'
	MOVWF	TMP
L18	CALL	DEL4X
	CALL	DEL4X
	DECFSZ	TMP,F
	GOTO	L18
	DECFSZ	OPRND,F
	GOTO	L17
L19	DECF	ACC,F
	HOME	PAD9

; Page 0 opcode support
P0PT1X	RDROM1	P0HI,P0LO
EAPT1X	RDROM1	EAHI,EALO
GOPX	GOP	P0PT1X,0
	RETURN
RD0X	GOP	P0PT1X,0
	GEA	P0HI,P0LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
DEL4X	RETURN
RD1X	GOP	P0PT1X,0
	GEA	P1HI,P1LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	RETURN
RD2X	GOP	P0PT1X,0
	GEA	P2HI,P2LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	RETURN
RD3X	GOP	P0PT1X,0
	GEA	P3HI,P3LO
	RDMEM	EAHI,EALO,DEL4X,EAPT1X,0
	RETURN
DADDX	DAD	DATUM
	RETURN
EA0X	CALL	GOPX
	GEA	P0HI,P0LO
	RETURN
EA1X	CALL	GOPX
	GEA	P1HI,P1LO
	RETURN
EA2X	CALL	GOPX
	GEA	P2HI,P2LO
	RETURN
EA3X	CALL	GOPX
	GEA	P3HI,P3LO
	RETURN
WREAX	WRMEM	EAHI,EALO
WR1X	WRMEM	P1HI,P1LO
WR2X	WRMEM	P2HI,P2LO
	ORG	H'0700'		; To prevent paging error
WR3X	WRMEM	P3HI,P3LO

; Only write to EEPROM if execution in SCIOS
;  and address is 0F12 or higher
CHKEE	BTFSC	TMP,7
	GOTO	WREE
	MOVLW	H'12'
	SUBWF	TMP,F
	BTFSC	TMP,7
	RETURN
	ADDWF	TMP,F
	GOTO	WREE

; Load from program memory
; Program number 0..7 expected in W
; Uses ACC, TMP and STAT
LDPM	ANDLW	B'00000111'
	ADDLW	HIGH PROG
	MOVWF	ACC
	CLRF	TMP
L13	CALL	L14
	CLRF	PCLATH
	MOVWF	STAT
	WRRAM	TMP,STAT
	INCFSZ	TMP,F
	GOTO	L13
	RETURN
L14	MOVF	ACC,W
	MOVWF	PCLATH
	MOVF	TMP,W
	MOVWF	PCL

; Page 1 - main emulation kernel (home page)
	ORG	H'0800'

; Opcode vector table
OPVEC	GOTO	XHALT	; 00 HALT    8~(SCMP)
	GOTO	XAE	; 01 XAE     7~
	GOTO	CCL	; 02 CCL     5~
	GOTO	SCL	; 03 SCL     5~
	GOTO	DINT	; 04 DINT    6~
	GOTO	IEN	; 05 IEN     6~
	GOTO	CSA	; 06 CSA     5~
	GOTO	CAS	; 07 CAS     6~
	GOTO	XNOP	; 08 NOP     5~
	GOTO	ILL	; 09
	GOTO	ILL	; 0A
	GOTO	ILL	; 0B
	GOTO	ILL	; 0C
	GOTO	ILL	; 0D
	GOTO	ILL	; 0E
	GOTO	ILL	; 0F
	GOTO	ILL	; 10
	GOTO	ILL	; 11
	GOTO	ILL	; 12
	GOTO	ILL	; 13
	GOTO	ILL	; 14
	GOTO	ILL	; 15
	GOTO	ILL	; 16
	GOTO	ILL	; 17
	GOTO	ILL	; 18
	GOTO	SIO	; 19 SIO     5~
	GOTO	ILL	; 1A
	GOTO	ILL	; 1B
	GOTO	SR	; 1C SR      5~  
	GOTO	SRL	; 1D SRL     5~
	GOTO	RR	; 1E RR      5~
	GOTO	RRL	; 1F RRL     5~
	GOTO	ILL	; 20
	GOTO	ILL	; 21
	GOTO	ILL	; 22
	GOTO	ILL	; 23
	GOTO	ILL	; 24
	GOTO	ILL	; 25
	GOTO	ILL	; 26
	GOTO	ILL	; 27
	GOTO	ILL	; 28
	GOTO	ILL	; 29
	GOTO	ILL	; 2A
	GOTO	ILL	; 2B
	GOTO	ILL	; 2C
	GOTO	ILL	; 2D
	GOTO	ILL	; 2E
	GOTO	ILL	; 2F
	GOTO	XPAL0	; 30 XPAL0   8~
	GOTO	XPAL1	; 31 XPAL1   8~  
	GOTO	XPAL2	; 32 XPAL2   8~
	GOTO	XPAL3	; 33 XPAL3   8~ 
	GOTO	XPAH0	; 34 XPAH0   8~
	GOTO	XPAH1	; 35 XPAH1   8~
	GOTO	XPAH2	; 36 XPAH2   8~
	GOTO	XPAH3	; 37 XPAH3   8~
	GOTO	ILL	; 38
	GOTO	ILL	; 39
	GOTO	ILL	; 3A
	GOTO	ILL	; 3B
	GOTO	XPPC0	; 3C XPPC0   7~
	GOTO	XPPC1	; 3D XPPC1   7~
	GOTO	XPPC2	; 3E XPPC2   7~
	GOTO	XPPC3	; 3F XPPC3   7~
	GOTO	LDE	; 40 LDE     6~
	GOTO	ILL	; 41
	GOTO	ILL	; 42
	GOTO	ILL	; 43
	GOTO	ILL	; 44
	GOTO	ILL	; 45
	GOTO	ILL	; 46
	GOTO	ILL	; 47
	GOTO	ILL	; 48
	GOTO	ILL	; 49
	GOTO	ILL	; 4A
	GOTO	ILL	; 4B
	GOTO	ILL	; 4C
	GOTO	ILL	; 4D
	GOTO	ILL	; 4E
	GOTO	ILL	; 4F
	GOTO	ANE	; 50 ANE     6~
	GOTO	ILL	; 51
	GOTO	ILL	; 52
	GOTO	ILL	; 53
	GOTO	ILL	; 54
	GOTO	ILL	; 55
	GOTO	ILL	; 56
	GOTO	ILL	; 57
	GOTO	ORE	; 58 ORE     6~
	GOTO	ILL	; 59
	GOTO	ILL	; 5A
	GOTO	ILL	; 5B
	GOTO	ILL	; 5C
	GOTO	ILL	; 5D
	GOTO	ILL	; 5E
	GOTO	ILL	; 5F
	GOTO	XRE	; 60 XRE     6~
	GOTO	ILL	; 61
	GOTO	ILL	; 62
	GOTO	ILL	; 63
	GOTO	ILL	; 64
	GOTO	ILL	; 65
	GOTO	ILL	; 66
	GOTO	ILL	; 67
	GOTO	DAE	; 68 DAE     11~
	GOTO	ILL	; 69
	GOTO	ILL	; 6A
	GOTO	ILL	; 6B
	GOTO	ILL	; 6C
	GOTO	ILL	; 6D
	GOTO	ILL	; 6E
	GOTO	ILL	; 6F
	GOTO	ADE	; 70 ADE     7~
	GOTO	ILL	; 71
	GOTO	ILL	; 72
	GOTO	ILL	; 73
	GOTO	ILL	; 74
	GOTO	ILL	; 75
	GOTO	ILL	; 76
	GOTO	ILL	; 77
	GOTO	CAE	; 78 CAE     8~
	GOTO	ILL	; 79
	GOTO	ILL	; 7A
	GOTO	ILL	; 7B
	GOTO	ILL	; 7C
	GOTO	ILL	; 7D
	GOTO	ILL	; 7E
	GOTO	ILL	; 7F
	GOTO	ILL	; 80
	GOTO	ILL	; 81
	GOTO	ILL	; 82
	GOTO	ILL	; 83
	GOTO	ILL	; 84
	GOTO	ILL	; 85
	GOTO	ILL	; 86
	GOTO	ILL	; 87
	GOTO	ILL	; 88
	GOTO	ILL	; 89
	GOTO	ILL	; 8A
	GOTO	ILL	; 8B
	GOTO	ILL	; 8C
	GOTO	ILL	; 8D
	GOTO	ILL	; 8E
	GOTO	DLY	; 8F DLY     13-131593~
	GOTO	JMP0	; 90 JMP0    11~
	GOTO	JMP1	; 91 JMP1    11~
	GOTO	JMP2	; 92 JMP2    11~
	GOTO	JMP3	; 93 JMP3    11~
	GOTO	JP0	; 94 JP0     9/11~
	GOTO	JP1	; 95 JP1     9/11~
	GOTO	JP2	; 96 JP2     9/11~
	GOTO	JP3	; 97 JP3     9/11~
	GOTO	JZ0	; 98 JZ0     9/11~
	GOTO	JZ1	; 99 JZ1     9/11~
	GOTO	JZ2	; 9A JZ2     9/11~
	GOTO	JZ3	; 9B JZ3     9/11~
	GOTO	JNZ0	; 9C JNZ0    9/11~
	GOTO	JNZ1	; 9D JNZ1    9/11~
	GOTO	JNZ2	; 9E JNZ2    9/11~
	GOTO	JNZ3	; 9F JNZ3    9/11~
	GOTO	ILL	; A0	
	GOTO	ILL	; A1
	GOTO	ILL	; A2
	GOTO	ILL	; A3
	GOTO	ILL	; A4
	GOTO	ILL	; A5
	GOTO	ILL	; A6
	GOTO	ILL	; A7
	GOTO	ILD0	; A8 ILD0    22~
	GOTO	ILD1	; A9 ILD1    22~
	GOTO	ILD2	; AA ILD2    22~
	GOTO	ILD3	; AB ILD3    22~
	GOTO	ILL	; AC
	GOTO	ILL	; AD
	GOTO	ILL	; AE
	GOTO	ILL	; AF
	GOTO	ILL	; B0
	GOTO	ILL	; B1
	GOTO	ILL	; B2
	GOTO	ILL	; B3
	GOTO	ILL	; B4
	GOTO	ILL	; B5
	GOTO	ILL	; B6
	GOTO	ILL	; B7
	GOTO	DLD0	; B8 DLD0    22~
	GOTO	DLD1	; B9 DLD1    22~
	GOTO	DLD2	; BA DLD2    22~
	GOTO	DLD3	; BB DLD3    22~
	GOTO	ILL	; BC
	GOTO	ILL	; BD
	GOTO	ILL	; BE
	GOTO	ILL	; BF
	GOTO	LD0	; C0 LD(0)   18~
	GOTO	LD1	; C1 LD(1)   18~
	GOTO	LD2	; C2 LD(2)   18~
	GOTO	LD3	; C3 LD(3)   18~
	GOTO	LDI	; C4 LDI     10~
	GOTO	LD1A	; C5 LD@(1)  18~
	GOTO	LD2A	; C6 LD@(2)  18~
	GOTO	LD3A	; C7 LD@(3)  18~
	GOTO	ST0	; C8 ST(0)   18~
	GOTO	ST1	; C9 ST(1)   18~
	GOTO	ST2	; CA ST(2)   18~
	GOTO	ST3	; CB ST(3)   18~
	GOTO	ILL	; CC
	GOTO	ST1A	; CD ST@(1)  18~
	GOTO	ST2A	; CE ST@(2)  18~
	GOTO	ST3A	; CF ST@(3)  18~  
	GOTO	AND0	; D0 AND(0)  18~
	GOTO	AND1	; D1 AND(1)  18~
	GOTO	AND2	; D2 AND(2)  18~
	GOTO	AND3	; D3 AND(3)  18~
	GOTO	ANI	; D4 ANI     10~
	GOTO	AND1A	; D5 AND@(1) 18~
	GOTO	AND2A	; D6 AND@(2) 18~
	GOTO	AND3A	; D7 AND@(3) 18~
	GOTO	OR0	; D8 OR(0)   18~
	GOTO	OR1	; D9 OR(1)   18~
	GOTO	OR2	; DA OR(2)   18~
	GOTO	OR3	; DB OR(3)   18~
	GOTO	ORI	; DC ORI     10~
	GOTO	OR1A	; DD OR@(1)  18~
	GOTO	OR2A	; DE OR@(2)  18~
	GOTO	OR3A	; DF OR@(3)  18~
	GOTO	XOR0	; E0 XOR(0)  18~
	GOTO	XOR1	; E1 XOR(1)  18~
	GOTO	XOR2	; E2 XOR(2)  18~
	GOTO	XOR3	; E3 XOR(3)  18~
	GOTO	XRI	; E4 XRI     10~
	GOTO	XOR1A	; E5 XOR@(1) 18~
	GOTO	XOR2A	; E6 XOR@(2) 18~
	GOTO	XOR3A	; E7 XOR@(3) 18~
	GOTO	DAD0	; E8 DAD(0)  23~
	GOTO	DAD1	; E9 DAD(1)  23~
	GOTO	DAD2	; EA DAD(2)  23~
	GOTO	DAD3	; EB DAD(3)  23~
	GOTO	DAI	; EC DAI     15~
	GOTO	DAD1A	; ED DAD@(1) 23~
	GOTO	DAD2A	; EE DAD@(2) 23~
	GOTO	DAD3A	; EF DAD@(3) 23~
	GOTO	ADD0	; F0 ADD(0)  19~
	GOTO	ADD1	; F1 ADD(1)  19~
	GOTO	ADD2	; F2 ADD(2)  19~
	GOTO	ADD3	; F3 ADD(3)  19~
	GOTO	ADI	; F4 ADI     11~
	GOTO	ADD1A	; F5 ADD@(1) 19~
	GOTO	ADD2A	; F6 ADD@(2) 19~
	GOTO	ADD3A	; F7 ADD@(3) 19~
	GOTO	CAD0	; F8 CAD(0)  20~
	GOTO	CAD1	; F9 CAD(1)  20~
	GOTO	CAD2	; FA CAD(2)  20~
	GOTO	CAD3	; FB CAD(3)  20~
	GOTO	CAI	; FC CAI     12~	
	GOTO	CAD1A	; FD CAD@(1) 20~
	GOTO	CAD2A	; FE CAD@(2) 20~
	GOTO	CAD3A	; FF CAD@(3) 20~

; Critical opcode - ADE
ADE	ADD	EXT

; Execute SCMP instruction (19)
; NB PCLATH must be set to HIGH OPVEC
EXEC	INCPC
	BTFSS	P0HI,3
	GOTO	L20
	RDRAM	P0LO
	MOVWF	PCL
L20	RDROM2	P0PT1,TMP,HIGH OPVEC
	MOVWF	PCL

; Opcode functions
CAE	MOVF	EXT,W
	MOVWF	DATUM
	COMF	DATUM,F
	ADD	DATUM
	GOTO	EXEC
RRL	RLF	STAT,F
	RRF	ACC,F
	RRF	STAT,F
	NOP
	GOTO	EXEC
LDE	MOVF	EXT,W
	MOVWF	ACC
	GOTO	PAD7
ORE	MOVF	EXT,W
	IORWF	ACC,F
	GOTO	PAD7
XRE	MOVF	EXT,W
	XORWF	ACC,F
	GOTO	PAD7
ANE	MOVF	EXT,W
	ANDWF	ACC,F
	GOTO	PAD7
SIO	RLF	PORTD,W
	RRF	EXT,F
	RLF	PORTE,F
	NOP
	GOTO	EXEC
SR	CLRF	TMP
	RRF	TMP,F
	RRF	ACC,F
	NOP
	GOTO	EXEC
RR	RRF	ACC,W
	RRF	ACC,F
	GOTO	PAD2
SRL	RLF	STAT,W
	RRF	ACC,F
	GOTO	PAD2
CCL	BCF	STAT,CYL
	GOTO	PAD3
SCL	BSF	STAT,CYL
	GOTO	PAD3
DINT	BCF	STAT,IE
	GOTO	PAD8
IEN	BSF	STAT,IE
	GOTO	PAD8
CAS	MOVF	ACC,W
	ANDLW	B'11001111'
	MOVWF	STAT
	MOVWF	PORTD
	GOTO	PAD5
CSA	MOVF	PORTD,W
	ANDLW	B'00110000'
	IORWF	STAT,W
	MOVWF	ACC
	GOTO	EXEC
XAE	LJMP0	XAEX
XPAL0	LJMP0	XPAL0X
XPAL1	LJMP0	XPAL1X
XPAL2	LJMP0	XPAL2X
XPAL3	LJMP0	XPAL3X
XPAH0	LJMP0	XPAH0X
XPAH1	LJMP0	XPAH1X
XPAH2	LJMP0	XPAH2X
XPAH3	LJMP0	XPAH3X
XPPC1	XCHNG	P1HI,P0HI
	XCHNG	P1LO,P0LO
	GOTO	PAD2
XPPC2	XCHNG	P2HI,P0HI
	XCHNG	P2LO,P0LO
	GOTO	PAD2
XPPC3	XCHNG	P3HI,P0HI
	XCHNG	P3LO,P0LO
	GOTO	PAD2
JMP0	NOP
	NOP
	GOTO	JUMP0
JMP1	NOP
	NOP
	GOTO	JUMP1
JMP2	NOP
	NOP
	GOTO	JUMP2
JMP3	NOP
	NOP
	GOTO	JUMP3
JP0	NOP
	BTFSS	ACC,7
	GOTO	JUMP0
	GOTO	NOJMP
JP1	NOP
	BTFSS	ACC,7
	GOTO	JUMP1
	GOTO	NOJMP
JP2	NOP
	BTFSS	ACC,7
	GOTO	JUMP2
	GOTO	NOJMP
JP3	NOP
	BTFSS	ACC,7
	GOTO	JUMP3
	GOTO	NOJMP
JZ0	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JUMP0
	GOTO	NOJMP
JZ1	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JUMP1
	GOTO	NOJMP
JZ2	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JUMP2
	GOTO	NOJMP
JZ3	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JUMP3
	GOTO	NOJMP
JNZ0	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JUMP0
	GOTO	NOJMP
JNZ1	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JUMP1
	GOTO	NOJMP
JNZ2	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JUMP2
	GOTO	NOJMP
JNZ3	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JUMP3
	GOTO	NOJMP
ILD0	LJMP0	ILD0X
ILD1	LJMP0	ILD1X
ILD2	LJMP0	ILD2X
ILD3	LJMP0	ILD3X
DLD0	LJMP0	DLD0X
DLD1	LJMP0	DLD1X
DLD2	LJMP0	DLD2X
DLD3	LJMP0	DLD3X
LD0	LJMP0	LD0X
LD1	LJMP0	LD1X
LD2	LJMP0	LD2X
LD3	LJMP0	LD3X
LD1A	CALL	RDA1
	MOVWF	ACC
	GOTO	PAD5
LD2A	CALL	RDA2
	MOVWF	ACC
	GOTO	PAD5
LD3A	CALL	RDA3
	MOVWF	ACC
	GOTO	PAD5
AND0	LJMP0	AND0X
AND1	LJMP0	AND1X
AND2	LJMP0	AND2X
AND3	LJMP0	AND3X
AND1A	CALL	RDA1
	ANDWF	ACC,F
	GOTO	PAD5
AND2A	CALL	RDA2
	ANDWF	ACC,F
	GOTO	PAD5
AND3A	CALL	RDA3
	ANDWF	ACC,F
	GOTO	PAD5
OR0	LJMP0	OR0X
OR1	LJMP0	OR1X
OR2	LJMP0	OR2X
OR3	LJMP0	OR3X
OR1A	CALL	RDA1
	IORWF	ACC,F
	GOTO	PAD5
OR2A	CALL	RDA2
	IORWF	ACC,F
	GOTO	PAD5
OR3A	CALL	RDA3
	IORWF	ACC,F
	GOTO	PAD5
XOR0	LJMP0	XOR0X
XOR1	LJMP0	XOR1X
XOR2	LJMP0	XOR2X
XOR3	LJMP0	XOR3X
XOR1A	CALL	RDA1
	XORWF	ACC,F
	GOTO	PAD5
XOR2A	CALL	RDA2
	XORWF	ACC,F
	GOTO	PAD5
XOR3A	CALL	RDA3
	XORWF	ACC,F
	GOTO	PAD5
DAD0	LJMP0	DAD0X
DAD1	LJMP0	DAD1X
DAD2	LJMP0	DAD2X
DAD3	LJMP0	DAD3X
DAD1A	CALL	RDA1
	CALL	DADD
	GOTO	PAD13
DAD2A	CALL	RDA2
	CALL	DADD
	GOTO	PAD13
DAD3A	CALL	RDA3
	CALL	DADD
	GOTO	PAD13
ADD0	LJMP0	ADD0X
ADD1	LJMP0	ADD1X
ADD2	LJMP0	ADD2X
ADD3	LJMP0	ADD3X
ADI	GOP	P0PT1,HIGH OPVEC
	ADD	OPRND
	NOP
	GOTO	EXEC
ADD1A	ADMEMA	P1HI,P1LO,P1PT1
ADD2A	ADMEMA	P2HI,P2LO,P2PT1
ADD3A	ADMEMA	P3HI,P3LO,P3PT1
CAD1A	RDMEMA	P1HI,P1LO,P1PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	PAD3
CAD2A	RDMEMA	P2HI,P2LO,P2PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	PAD3
CAD3A	RDMEMA	P3HI,P3LO,P3PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	PAD3
DAE	LJMP0	DAEX
DLY	LJMP0	DLYX
LDI	LJMP0	LDIX
ST0	LJMP0	ST0X
ST1	LJMP0	ST1X
ST2	LJMP0	ST2X
ST3	LJMP0	ST3X
ST1A	LJMP0	ST1AX
ST2A	LJMP0	ST2AX
ST3A	LJMP0	ST3AX
ANI	LJMP0	ANIX
ORI	LJMP0	ORIX
XRI	LJMP0	XRIX
DAI	LJMP0	DAIX
CAD0	LJMP0	CAD0X
CAD1	LJMP0	CAD1X
CAD2	LJMP0	CAD2X
CAD3	LJMP0	CAD3X
CAI	LJMP0	CAIX

; Opcode support
P0PT1	RDROM1	P0HI,P0LO
P1PT1	RDROM1	P1HI,P1LO
P2PT1	RDROM1	P2HI,P2LO
P3PT1	RDROM1	P3HI,P3LO
EAPT1	RDROM1	EAHI,EALO
JUMP0	GEAPC	P0HI,P0LO
JUMP1	GEAPC	P1HI,P1LO
JUMP2	GEAPC	P2HI,P2LO
JUMP3	GEAPC	P3HI,P3LO
NOJMP	INCPC
	GOTO	PAD16
RDA1	RDMEMA	P1HI,P1LO,P1PT1
	RETURN
RDA2	RDMEMA	P2HI,P2LO,P2PT1
	RETURN
RDA3	RDMEMA	P3HI,P3LO,P3PT1
DEL4	RETURN
DADD	DAD	DATUM
	RETURN
XHALT
PAD21	CALL	DEL4
PAD17	NOP
XPPC0
PAD16	NOP
PAD15	NOP
PAD14	NOP
PAD13	NOP
PAD12	NOP
PAD11	NOP
PAD10	NOP
PAD9	NOP
PAD8	NOP
PAD7	NOP
XNOP
PAD6	NOP
PAD5	NOP
PAD4	NOP
PAD3	BTFSC	PORTA,RST
PAD2	GOTO	EXEC
ILL	LJMP0	RESET

; Page 2 - SCIOS ROM table
	ORG	H'1000'
SCIOS	DT	H'00', H'CF', H'FF', H'90', H'1E', H'37', H'C2', H'0C'
	DT	H'33', H'C7', H'FF', H'C0', H'F2', H'01', H'C0', H'EB'
	DT	H'31', H'C0', H'E7', H'35', H'C0', H'E7', H'32', H'C0'
	DT	H'E3', H'36', H'C0', H'E4', H'00', H'07', H'C0', H'DE'
	DT	H'08', H'05', H'3F', H'C8', H'D9', H'40', H'C8', H'D7'
	DT	H'06', H'C8', H'D5', H'35', H'C8', H'CC', H'31', H'C8'
	DT	H'CA', H'C4', H'0F', H'36', H'C8', H'C6', H'C4', H'00'
	DT	H'32', H'C8', H'C2', H'C7', H'01', H'33', H'CA', H'0C'
	DT	H'37', H'CA', H'0E', H'C4', H'00', H'CA', H'02', H'CA'
	DT	H'03', H'C4', H'01', H'37', H'90', H'6D', H'C2', H'0E'
	DT	H'90', H'B3', H'C5', H'01', H'01', H'C4', H'01', H'CB'
	DT	H'D5', H'C4', H'01', H'07', H'8F', H'08', H'C3', H'D5'
	DT	H'50', H'98', H'07', H'8F', H'18', H'C4', H'00', H'07'
	DT	H'90', H'05', H'C4', H'00', H'07', H'8F', H'18', H'8F'
	DT	H'20', H'C3', H'D5', H'F3', H'D5', H'9C', H'E0', H'BB'
	DT	H'D6', H'9C', H'D7', H'3F', H'C4', H'08', H'CB', H'D5'
	DT	H'06', H'D4', H'20', H'98', H'FB', H'8F', H'1C', H'19'
	DT	H'8F', H'1C', H'BB', H'D5', H'9C', H'F2', H'40', H'CD'
	DT	H'01', H'90', H'E9', H'C6', H'FE', H'32', H'03', H'FB'
	DT	H'D8', H'C9', H'01', H'3F', H'08', H'AA', H'0E', H'90'
	DT	H'36', H'C2', H'0E', H'35', H'C2', H'0C', H'31', H'C2'
	DT	H'0D', H'C9', H'00', H'90', H'34', H'E4', H'06', H'98'
	DT	H'9D', H'E4', H'05', H'98', H'22', H'AA', H'0C', H'9C'
	DT	H'1E', H'90', H'E2', H'C4', H'FF', H'CA', H'11', H'CA'
	DT	H'0F', H'C2', H'0E', H'35', H'C2', H'0C', H'31', H'C1'
	DT	H'00', H'CA', H'0D', H'C4', H'3F', H'33', H'3F', H'90'
	DT	H'DC', H'C4', H'1A', H'33', H'3F', H'90', H'EA', H'C4'
	DT	H'FF', H'CA', H'0F', H'C2', H'0E', H'35', H'C2', H'0C'
	DT	H'31', H'C1', H'00', H'CA', H'0D', H'C4', H'3F', H'33'
	DT	H'3F', H'90', H'C2', H'C4', H'04', H'CA', H'09', H'AA'
	DT	H'0F', H'9C', H'06', H'C4', H'00', H'CA', H'0D', H'CA'
	DT	H'11', H'02', H'C2', H'0D', H'F2', H'0D', H'CA', H'0D'
	DT	H'BA', H'09', H'9C', H'F5', H'C2', H'0D', H'58', H'CA'
	DT	H'0D', H'90', H'96', H'3F', H'06', H'5B', H'4F', H'66'
	DT	H'6D', H'7D', H'07', H'7F', H'67', H'77', H'7C', H'39'
	DT	H'5E', H'79', H'71', H'C4', H'04', H'CA', H'09', H'AA'
	DT	H'0F', H'9C', H'06', H'C4', H'00', H'CA', H'0E', H'CA'
	DT	H'0C', H'02', H'C2', H'0C', H'F2', H'0C', H'CA', H'0C'
	DT	H'C2', H'0E', H'F2', H'0E', H'CA', H'0E', H'BA', H'09'
	DT	H'9C', H'EF', H'C2', H'0C', H'58', H'CA', H'0C', H'3F'
	DT	H'C4', H'01', H'35', H'C4', H'0B', H'31', H'C2', H'0D'
	DT	H'D4', H'0F', H'01', H'C1', H'80', H'CA', H'00', H'C2'
	DT	H'0D', H'1C', H'1C', H'1C', H'1C', H'01', H'C1', H'80'
	DT	H'CA', H'01', H'03', H'C4', H'01', H'35', H'C4', H'0B'
	DT	H'31', H'C2', H'0C', H'D4', H'0F', H'01', H'C1', H'80'
	DT	H'CA', H'04', H'C2', H'0C', H'1C', H'1C', H'1C', H'1C'
	DT	H'01', H'C1', H'80', H'CA', H'05', H'06', H'D4', H'80'
	DT	H'98', H'09', H'02', H'C4', H'00', H'CA', H'03', H'C6'
	DT	H'02', H'90', H'DE', H'C6', H'FE', H'C4', H'00', H'CA'
	DT	H'0B', H'C4', H'0D', H'35', H'C4', H'FF', H'CA', H'10'
	DT	H'C4', H'0A', H'CA', H'09', H'C4', H'00', H'CA', H'0A'
	DT	H'31', H'AA', H'10', H'01', H'C2', H'80', H'C9', H'80'
	DT	H'8F', H'00', H'C1', H'80', H'E4', H'FF', H'9C', H'4C'
	DT	H'BA', H'09', H'9C', H'ED', H'C2', H'0A', H'98', H'0A'
	DT	H'C2', H'0B', H'9C', H'D8', H'C2', H'0A', H'CA', H'0B'
	DT	H'90', H'D2', H'C2', H'0B', H'98', H'CE', H'01', H'40'
	DT	H'D4', H'20', H'9C', H'28', H'C4', H'80', H'50', H'9C'
	DT	H'1B', H'C4', H'40', H'50', H'9C', H'19', H'C4', H'0F'
	DT	H'50', H'F4', H'07', H'01', H'C0', H'80', H'01', H'C7'
	DT	H'02', H'3F', H'90', H'A9', H'0A', H'0B', H'0C', H'0D'
	DT	H'00', H'00', H'0E', H'0F', H'60', H'90', H'EF', H'60'
	DT	H'F4', H'08', H'90', H'EA', H'60', H'E4', H'04', H'98'
	DT	H'08', H'3F', H'90', H'91', H'58', H'CA', H'0A', H'90'
	DT	H'AF', H'C4', H'00', H'37', H'C4', H'4B', H'33', H'3F'

; Program images
	ORG	0x1800
PROG

	END
